@@ -1,5 +1,6 @@
 /*
     SPDX-FileCopyrightText: 2019 Roman Gilg <subdiff@gmail.com>
+    SPDX-FileCopyrightText: 2021 Xaver Hugl <xaver.hugl@gmail.com>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
@@ -14,6 +15,22 @@
 
 #include <kscreen/config.h>
 
+// clang-format off
+#define retentionString                 QStringLiteral("retention")
+#define nameString                      QStringLiteral("name")
+#define scaleString                     QStringLiteral("scale")
+#define metadataString                  QStringLiteral("metadata")
+#define idString                        QStringLiteral("id")
+#define autorotateString                QStringLiteral("autorotate")
+#define autorotateTabletOnlyString      QStringLiteral("autorotate-tablet-only")
+#define replicateHashString             QStringLiteral("replicate-hash")
+#define replicateNameString             QStringLiteral("replicate-name")
+#define overscanString                  QStringLiteral("overscan")
+#define vrrPolicyString                 QStringLiteral("vrrpolicy")
+#define rgbRangeString                  QStringLiteral("rgbrange")
+#define outputsString                   QStringLiteral("outputs")
+// clang-format on
+
 QString Control::s_dirName = QStringLiteral("control/");
 
 Control::Control(QObject *parent)
@@ -134,7 +151,7 @@ ControlConfig::ControlConfig(KScreen::ConfigPtr config, QObject *parent)
         allIds << outputId;
     }
 
-    for (auto output : outputs) {
+    for (const auto &output : outputs) {
         m_outputsControls << new ControlOutput(output, this);
     }
 
@@ -150,7 +167,7 @@ void ControlConfig::activateWatcher()
         // Watcher was already activated.
         return;
     }
-    for (auto *output : m_outputsControls) {
+    for (auto *output : qAsConst(m_outputsControls)) {
         output->activateWatcher();
         connect(output, &ControlOutput::changed, this, &ControlConfig::changed);
     }
@@ -172,7 +189,7 @@ QString ControlConfig::filePath() const
 bool ControlConfig::writeFile()
 {
     bool success = true;
-    for (auto *outputControl : m_outputsControls) {
+    for (auto *outputControl : qAsConst(m_outputsControls)) {
         if (getOutputRetention(outputControl->id(), outputControl->name()) == OutputRetention::Individual) {
             continue;
         }
@@ -183,7 +200,7 @@ bool ControlConfig::writeFile()
 
 bool ControlConfig::infoIsOutput(const QVariantMap &info, const QString &outputId, const QString &outputName) const
 {
-    const QString outputIdInfo = info[QStringLiteral("id")].toString();
+    const QString outputIdInfo = info[idString].toString();
     if (outputIdInfo.isEmpty()) {
         return false;
     }
@@ -194,8 +211,8 @@ bool ControlConfig::infoIsOutput(const QVariantMap &info, const QString &outputI
     if (!outputName.isEmpty() && m_duplicateOutputIds.contains(outputId)) {
         // We may have identical outputs connected, these will have the same id in the config
         // in order to find the right one, also check the output's name (usually the connector)
-        const auto metadata = info[QStringLiteral("metadata")].toMap();
-        const auto outputNameInfo = metadata[QStringLiteral("name")].toString();
+        const auto metadata = info[metadataString].toMap();
+        const auto outputNameInfo = metadata[nameString].toString();
         if (outputName != outputNameInfo) {
             // was a duplicate id, but info not for this output
             return false;
@@ -217,7 +234,7 @@ Control::OutputRetention ControlConfig::getOutputRetention(const QString &output
         if (!infoIsOutput(info, outputId, outputName)) {
             continue;
         }
-        return convertVariantToOutputRetention(info[QStringLiteral("retention")]);
+        return convertVariantToOutputRetention(info[retentionString]);
     }
     // info for output not found
     return OutputRetention::Undefined;
@@ -226,15 +243,15 @@ Control::OutputRetention ControlConfig::getOutputRetention(const QString &output
 static QVariantMap metadata(const QString &outputName)
 {
     QVariantMap metadata;
-    metadata[QStringLiteral("name")] = outputName;
+    metadata[nameString] = outputName;
     return metadata;
 }
 
 QVariantMap createOutputInfo(const QString &outputId, const QString &outputName)
 {
     QVariantMap outputInfo;
-    outputInfo[QStringLiteral("id")] = outputId;
-    outputInfo[QStringLiteral("metadata")] = metadata(outputName);
+    outputInfo[idString] = outputId;
+    outputInfo[metadataString] = metadata(outputName);
     return outputInfo;
 }
 
@@ -253,26 +270,24 @@ void ControlConfig::setOutputRetention(const QString &outputId, const QString &o
         if (!infoIsOutput(outputInfo, outputId, outputName)) {
             continue;
         }
-        outputInfo[QStringLiteral("retention")] = (int)value;
+        outputInfo[retentionString] = (int)value;
         *it = outputInfo;
         setOutputs(outputsInfo);
         return;
     }
     // no entry yet, create one
     auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("retention")] = (int)value;
+    outputInfo[retentionString] = (int)value;
 
     outputsInfo << outputInfo;
     setOutputs(outputsInfo);
 }
 
-qreal ControlConfig::getScale(const KScreen::OutputPtr &output) const
-{
-    return getScale(output->hashMd5(), output->name());
-}
-
-qreal ControlConfig::getScale(const QString &outputId, const QString &outputName) const
+template<typename T, typename F>
+T ControlConfig::get(const KScreen::OutputPtr &output, const QString &name, F globalRetentionFunc, T defaultValue) const
 {
+    const auto &outputId = output->hashMd5();
+    const auto &outputName = output->name();
     const auto retention = getOutputRetention(outputId, outputName);
     if (retention == OutputRetention::Individual) {
         const QVariantList outputsInfo = getOutputs();
@@ -281,208 +296,93 @@ qreal ControlConfig::getScale(const QString &outputId, const QString &outputName
             if (!infoIsOutput(info, outputId, outputName)) {
                 continue;
             }
-            const auto val = info[QStringLiteral("scale")];
-            return val.canConvert<qreal>() ? val.toReal() : -1;
+            const auto val = info[name];
+            if (val.template canConvert<T>()) {
+                return val.template value<T>();
+            } else {
+                return defaultValue;
+            }
         }
     }
     // Retention is global or info for output not in config control file.
     if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->getScale();
+        return (outputControl->*globalRetentionFunc)();
     }
 
     // Info for output not found.
-    return -1;
+    return defaultValue;
 }
 
-void ControlConfig::setScale(const KScreen::OutputPtr &output, qreal value)
-{
-    setScale(output->hashMd5(), output->name(), value);
-}
-
-// TODO: combine methods (templated functions)
-void ControlConfig::setScale(const QString &outputId, const QString &outputName, qreal value)
+template<typename T, typename F, typename V>
+void ControlConfig::set(const KScreen::OutputPtr &output, const QString &name, F globalRetentionFunc, V value)
 {
+    const auto &outputId = output->hashMd5();
+    const auto &outputName = output->name();
     QList<QVariant>::iterator it;
     QVariantList outputsInfo = getOutputs();
 
-    auto setOutputScale = [&outputId, &outputName, value, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setScale(value);
-        }
-    };
-
     for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
         QVariantMap outputInfo = (*it).toMap();
         if (!infoIsOutput(outputInfo, outputId, outputName)) {
             continue;
         }
-        outputInfo[QStringLiteral("scale")] = value;
+        outputInfo[name] = static_cast<T>(value);
         *it = outputInfo;
         setOutputs(outputsInfo);
-        setOutputScale();
+        if (auto *control = getOutputControl(outputId, outputName)) {
+            (control->*globalRetentionFunc)(value);
+        }
         return;
     }
     // no entry yet, create one
     auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("scale")] = value;
+    outputInfo[name] = static_cast<T>(value);
 
     outputsInfo << outputInfo;
     setOutputs(outputsInfo);
-    setOutputScale();
+    if (auto *control = getOutputControl(outputId, outputName)) {
+        (control->*globalRetentionFunc)(value);
+    }
 }
 
 bool ControlConfig::getAutoRotate(const KScreen::OutputPtr &output) const
 {
-    return getAutoRotate(output->hashMd5(), output->name());
-}
-
-bool ControlConfig::getAutoRotate(const QString &outputId, const QString &outputName) const
-{
-    const auto retention = getOutputRetention(outputId, outputName);
-    if (retention == OutputRetention::Individual) {
-        const QVariantList outputsInfo = getOutputs();
-        for (const auto &variantInfo : outputsInfo) {
-            const QVariantMap info = variantInfo.toMap();
-            if (!infoIsOutput(info, outputId, outputName)) {
-                continue;
-            }
-            const auto val = info[QStringLiteral("autorotate")];
-            return !val.canConvert<bool>() || val.toBool();
-        }
-    }
-    // Retention is global or info for output not in config control file.
-    if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->getAutoRotate();
-    }
-
-    // Info for output not found.
-    return true;
+    return get(output, autorotateString, &ControlOutput::getAutoRotate, true);
 }
 
 void ControlConfig::setAutoRotate(const KScreen::OutputPtr &output, bool value)
 {
-    setAutoRotate(output->hashMd5(), output->name(), value);
-}
-
-// TODO: combine methods (templated functions)
-void ControlConfig::setAutoRotate(const QString &outputId, const QString &outputName, bool value)
-{
-    QList<QVariant>::iterator it;
-    QVariantList outputsInfo = getOutputs();
-
-    auto setOutputAutoRotate = [&outputId, &outputName, value, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setAutoRotate(value);
-        }
-    };
-
-    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
-        QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
-            continue;
-        }
-        outputInfo[QStringLiteral("autorotate")] = value;
-        *it = outputInfo;
-        setOutputs(outputsInfo);
-        setOutputAutoRotate();
-        return;
-    }
-    // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("autorotate")] = value;
-
-    outputsInfo << outputInfo;
-    setOutputs(outputsInfo);
-    setOutputAutoRotate();
+    set<bool>(output, autorotateString, &ControlOutput::setAutoRotate, value);
 }
 
 bool ControlConfig::getAutoRotateOnlyInTabletMode(const KScreen::OutputPtr &output) const
 {
-    return getAutoRotateOnlyInTabletMode(output->hashMd5(), output->name());
-}
-
-bool ControlConfig::getAutoRotateOnlyInTabletMode(const QString &outputId, const QString &outputName) const
-{
-    const auto retention = getOutputRetention(outputId, outputName);
-    if (retention == OutputRetention::Individual) {
-        const QVariantList outputsInfo = getOutputs();
-        for (const auto &variantInfo : outputsInfo) {
-            const QVariantMap info = variantInfo.toMap();
-            if (!infoIsOutput(info, outputId, outputName)) {
-                continue;
-            }
-            const auto val = info[QStringLiteral("autorotate-tablet-only")];
-            return !val.canConvert<bool>() || val.toBool();
-        }
-    }
-    // Retention is global or info for output not in config control file.
-    if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->getAutoRotateOnlyInTabletMode();
-    }
-
-    // Info for output not found.
-    return true;
+    return get(output, autorotateTabletOnlyString, &ControlOutput::getAutoRotateOnlyInTabletMode, true);
 }
 
 void ControlConfig::setAutoRotateOnlyInTabletMode(const KScreen::OutputPtr &output, bool value)
 {
-    setAutoRotateOnlyInTabletMode(output->hashMd5(), output->name(), value);
-}
-
-// TODO: combine methods (templated functions)
-void ControlConfig::setAutoRotateOnlyInTabletMode(const QString &outputId, const QString &outputName, bool value)
-{
-    QList<QVariant>::iterator it;
-    QVariantList outputsInfo = getOutputs();
-
-    auto setOutputAutoRotateOnlyInTabletMode = [&outputId, &outputName, value, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setAutoRotateOnlyInTabletMode(value);
-        }
-    };
-
-    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
-        QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
-            continue;
-        }
-        outputInfo[QStringLiteral("autorotate-tablet-only")] = value;
-        *it = outputInfo;
-        setOutputs(outputsInfo);
-        setOutputAutoRotateOnlyInTabletMode();
-        return;
-    }
-    // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("autorotate-tablet-only")] = value;
-
-    outputsInfo << outputInfo;
-    setOutputs(outputsInfo);
-    setOutputAutoRotateOnlyInTabletMode();
+    set<bool>(output, autorotateTabletOnlyString, &ControlOutput::setAutoRotateOnlyInTabletMode, value);
 }
 
 KScreen::OutputPtr ControlConfig::getReplicationSource(const KScreen::OutputPtr &output) const
-{
-    return getReplicationSource(output->hashMd5(), output->name());
-}
-
-KScreen::OutputPtr ControlConfig::getReplicationSource(const QString &outputId, const QString &outputName) const
 {
     const QVariantList outputsInfo = getOutputs();
     for (const auto &variantInfo : outputsInfo) {
         const QVariantMap info = variantInfo.toMap();
-        if (!infoIsOutput(info, outputId, outputName)) {
+        if (!infoIsOutput(info, output->hashMd5(), output->name())) {
             continue;
         }
-        const QString sourceHash = info[QStringLiteral("replicate-hash")].toString();
-        const QString sourceName = info[QStringLiteral("replicate-name")].toString();
+        const QString sourceHash = info[replicateHashString].toString();
+        const QString sourceName = info[replicateNameString].toString();
 
         if (sourceHash.isEmpty() && sourceName.isEmpty()) {
             // Common case when the replication source has been unset.
             return nullptr;
         }
 
-        for (const auto &output : m_config->outputs()) {
+        const auto outputs = m_config->outputs();
+        for (const auto &output : outputs) {
             if (output->hashMd5() == sourceHash && output->name() == sourceName) {
                 return output;
             }
@@ -495,33 +395,28 @@ KScreen::OutputPtr ControlConfig::getReplicationSource(const QString &outputId,
 }
 
 void ControlConfig::setReplicationSource(const KScreen::OutputPtr &output, const KScreen::OutputPtr &source)
-{
-    setReplicationSource(output->hashMd5(), output->name(), source);
-}
-
-void ControlConfig::setReplicationSource(const QString &outputId, const QString &outputName, const KScreen::OutputPtr &source)
 {
     QList<QVariant>::iterator it;
     QVariantList outputsInfo = getOutputs();
-    const QString sourceHash = source ? source->hashMd5() : QStringLiteral("");
-    const QString sourceName = source ? source->name() : QStringLiteral("");
+    const QString sourceHash = source ? source->hashMd5() : QString();
+    const QString sourceName = source ? source->name() : QString();
 
     for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
         QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
+        if (!infoIsOutput(outputInfo, output->hashMd5(), output->name())) {
             continue;
         }
-        outputInfo[QStringLiteral("replicate-hash")] = sourceHash;
-        outputInfo[QStringLiteral("replicate-name")] = sourceName;
+        outputInfo[replicateHashString] = sourceHash;
+        outputInfo[replicateNameString] = sourceName;
         *it = outputInfo;
         setOutputs(outputsInfo);
         // TODO: shall we set this information also as new global value (like with auto-rotate)?
         return;
     }
     // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("replicate-hash")] = sourceHash;
-    outputInfo[QStringLiteral("replicate-name")] = sourceName;
+    auto outputInfo = createOutputInfo(output->hashMd5(), output->name());
+    outputInfo[replicateHashString] = sourceHash;
+    outputInfo[replicateNameString] = sourceName;
 
     outputsInfo << outputInfo;
     setOutputs(outputsInfo);
@@ -530,220 +425,45 @@ void ControlConfig::setReplicationSource(const QString &outputId, const QString
 
 uint32_t ControlConfig::getOverscan(const KScreen::OutputPtr &output) const
 {
-    return getOverscan(output->hashMd5(), output->name());
-}
-
-uint32_t ControlConfig::getOverscan(const QString &outputId, const QString &outputName) const
-{
-    const auto retention = getOutputRetention(outputId, outputName);
-    if (retention == OutputRetention::Individual) {
-        const QVariantList outputsInfo = getOutputs();
-        for (const auto &variantInfo : outputsInfo) {
-            const QVariantMap info = variantInfo.toMap();
-            if (!infoIsOutput(info, outputId, outputName)) {
-                continue;
-            }
-            const auto val = info[QStringLiteral("overscan")];
-            if (val.canConvert<uint>()) {
-                return static_cast<uint32_t>(val.toUInt());
-            } else {
-                return 0;
-            }
-        }
-    }
-    // Retention is global or info for output not in config control file.
-    if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->overscan();
-    }
-
-    // Info for output not found.
-    return 0;
+    return get(output, overscanString, &ControlOutput::overscan, 0);
 }
 
 void ControlConfig::setOverscan(const KScreen::OutputPtr &output, const uint32_t value)
 {
-    setOverscan(output->hashMd5(), output->name(), value);
-}
-
-void ControlConfig::setOverscan(const QString &outputId, const QString &outputName, const uint32_t value)
-{
-    QList<QVariant>::iterator it;
-    QVariantList outputsInfo = getOutputs();
-
-    auto setOutputOverscan = [&outputId, &outputName, value, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setOverscan(value);
-        }
-    };
-
-    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
-        QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
-            continue;
-        }
-        outputInfo[QStringLiteral("overscan")] = value;
-        *it = outputInfo;
-        setOutputs(outputsInfo);
-        setOutputOverscan();
-        return;
-    }
-    // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("overscan")] = value;
-
-    outputsInfo << outputInfo;
-    setOutputs(outputsInfo);
-    setOutputOverscan();
+    set<uint32_t>(output, overscanString, &ControlOutput::setOverscan, value);
 }
 
 KScreen::Output::VrrPolicy ControlConfig::getVrrPolicy(const KScreen::OutputPtr &output) const
 {
-    return getVrrPolicy(output->hashMd5(), output->name());
-}
-
-KScreen::Output::VrrPolicy ControlConfig::getVrrPolicy(const QString &outputId, const QString &outputName) const
-{
-    const auto retention = getOutputRetention(outputId, outputName);
-    if (retention == OutputRetention::Individual) {
-        const QVariantList outputsInfo = getOutputs();
-        for (const auto &variantInfo : outputsInfo) {
-            const QVariantMap info = variantInfo.toMap();
-            if (!infoIsOutput(info, outputId, outputName)) {
-                continue;
-            }
-            const auto val = info[QStringLiteral("vrrpolicy")];
-            if (val.canConvert<uint>()) {
-                return static_cast<KScreen::Output::VrrPolicy>(val.toUInt());
-            } else {
-                return KScreen::Output::VrrPolicy::Automatic;
-            }
-        }
-    }
-    // Retention is global or info for output not in config control file.
-    if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->vrrPolicy();
-    }
-
-    // Info for output not found.
-    return KScreen::Output::VrrPolicy::Automatic;
+    return get(output, vrrPolicyString, &ControlOutput::vrrPolicy, KScreen::Output::VrrPolicy::Automatic);
 }
 
 void ControlConfig::setVrrPolicy(const KScreen::OutputPtr &output, const KScreen::Output::VrrPolicy value)
 {
-    setVrrPolicy(output->hashMd5(), output->name(), value);
-}
-
-void ControlConfig::setVrrPolicy(const QString &outputId, const QString &outputName, const KScreen::Output::VrrPolicy enumValue)
-{
-    QList<QVariant>::iterator it;
-    QVariantList outputsInfo = getOutputs();
-    uint value = static_cast<uint>(enumValue);
-
-    auto setOutputVrr = [&outputId, &outputName, enumValue, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setVrrPolicy(enumValue);
-        }
-    };
-
-    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
-        QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
-            continue;
-        }
-        outputInfo[QStringLiteral("vrrpolicy")] = value;
-        *it = outputInfo;
-        setOutputs(outputsInfo);
-        setOutputVrr();
-        return;
-    }
-    // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("vrrpolicy")] = value;
-
-    outputsInfo << outputInfo;
-    setOutputs(outputsInfo);
-    setOutputVrr();
+    set<uint32_t>(output, vrrPolicyString, &ControlOutput::setVrrPolicy, value);
 }
 
 KScreen::Output::RgbRange ControlConfig::getRgbRange(const KScreen::OutputPtr &output) const
 {
-    return getRgbRange(output->hashMd5(), output->name());
-}
-
-KScreen::Output::RgbRange ControlConfig::getRgbRange(const QString &outputId, const QString &outputName) const
-{
-    const auto retention = getOutputRetention(outputId, outputName);
-    if (retention == OutputRetention::Individual) {
-        const QVariantList outputsInfo = getOutputs();
-        for (const auto &variantInfo : outputsInfo) {
-            const QVariantMap info = variantInfo.toMap();
-            if (!infoIsOutput(info, outputId, outputName)) {
-                continue;
-            }
-            const auto val = info[QStringLiteral("rgbrange")];
-            if (val.canConvert<uint>()) {
-                return static_cast<KScreen::Output::RgbRange>(val.toUInt());
-            } else {
-                return KScreen::Output::RgbRange::Automatic;
-            }
-        }
-    }
-    // Retention is global or info for output not in config control file.
-    if (auto *outputControl = getOutputControl(outputId, outputName)) {
-        return outputControl->rgbRange();
-    }
-
-    // Info for output not found.
-    return KScreen::Output::RgbRange::Automatic;
+    return get(output, rgbRangeString, &ControlOutput::rgbRange, KScreen::Output::RgbRange::Automatic);
 }
 
 void ControlConfig::setRgbRange(const KScreen::OutputPtr &output, const KScreen::Output::RgbRange value)
 {
-    setRgbRange(output->hashMd5(), output->name(), value);
-}
-
-void ControlConfig::setRgbRange(const QString &outputId, const QString &outputName, const KScreen::Output::RgbRange enumValue)
-{
-    QList<QVariant>::iterator it;
-    QVariantList outputsInfo = getOutputs();
-    uint value = static_cast<uint>(enumValue);
-
-    auto setOutputRgbRange = [&outputId, &outputName, enumValue, this]() {
-        if (auto *control = getOutputControl(outputId, outputName)) {
-            control->setRgbRange(enumValue);
-        }
-    };
-
-    for (it = outputsInfo.begin(); it != outputsInfo.end(); ++it) {
-        QVariantMap outputInfo = (*it).toMap();
-        if (!infoIsOutput(outputInfo, outputId, outputName)) {
-            continue;
-        }
-        outputInfo[QStringLiteral("rgbrange")] = value;
-        *it = outputInfo;
-        setOutputs(outputsInfo);
-        setOutputRgbRange();
-        return;
-    }
-    // no entry yet, create one
-    auto outputInfo = createOutputInfo(outputId, outputName);
-    outputInfo[QStringLiteral("rgbrange")] = value;
-
-    outputsInfo << outputInfo;
-    setOutputs(outputsInfo);
-    setOutputRgbRange();
+    set<uint32_t>(output, rgbRangeString, &ControlOutput::setRgbRange, value);
 }
 
 QVariantList ControlConfig::getOutputs() const
 {
-    return constInfo()[QStringLiteral("outputs")].toList();
+    return constInfo()[outputsString].toList();
 }
 
 void ControlConfig::setOutputs(QVariantList outputsInfo)
 {
     auto &infoMap = info();
-    infoMap[QStringLiteral("outputs")] = outputsInfo;
+    infoMap[outputsString] = outputsInfo;
 }
+
 ControlOutput *ControlConfig::getOutputControl(const QString &outputId, const QString &outputName) const
 {
     for (auto *control : m_outputsControls) {
@@ -784,24 +504,9 @@ QString ControlOutput::filePath() const
     return filePathFromHash(m_output->hashMd5());
 }
 
-qreal ControlOutput::getScale() const
-{
-    const auto val = constInfo()[QStringLiteral("scale")];
-    return val.canConvert<qreal>() ? val.toReal() : -1;
-}
-
-void ControlOutput::setScale(qreal value)
-{
-    auto &infoMap = info();
-    if (infoMap.isEmpty()) {
-        infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
-    }
-    infoMap[QStringLiteral("scale")] = value;
-}
-
 bool ControlOutput::getAutoRotate() const
 {
-    const auto val = constInfo()[QStringLiteral("autorotate")];
+    const auto val = constInfo()[autorotateString];
     return !val.canConvert<bool>() || val.toBool();
 }
 
@@ -811,12 +516,12 @@ void ControlOutput::setAutoRotate(bool value)
     if (infoMap.isEmpty()) {
         infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
     }
-    infoMap[QStringLiteral("autorotate")] = value;
+    infoMap[autorotateString] = value;
 }
 
 bool ControlOutput::getAutoRotateOnlyInTabletMode() const
 {
-    const auto val = constInfo()[QStringLiteral("autorotate-tablet-only")];
+    const auto val = constInfo()[autorotateTabletOnlyString];
     return !val.canConvert<bool>() || val.toBool();
 }
 
@@ -826,12 +531,12 @@ void ControlOutput::setAutoRotateOnlyInTabletMode(bool value)
     if (infoMap.isEmpty()) {
         infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
     }
-    infoMap[QStringLiteral("autorotate-tablet-only")] = value;
+    infoMap[autorotateTabletOnlyString] = value;
 }
 
 uint32_t ControlOutput::overscan() const
 {
-    const auto val = constInfo()[QStringLiteral("overscan")];
+    const auto val = constInfo()[overscanString];
     if (val.canConvert<uint>()) {
         return val.toUInt();
     }
@@ -844,12 +549,12 @@ void ControlOutput::setOverscan(uint32_t value)
     if (infoMap.isEmpty()) {
         infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
     }
-    infoMap[QStringLiteral("overscan")] = static_cast<uint>(value);
+    infoMap[overscanString] = static_cast<uint>(value);
 }
 
 KScreen::Output::VrrPolicy ControlOutput::vrrPolicy() const
 {
-    const auto val = constInfo()[QStringLiteral("vrrpolicy")];
+    const auto val = constInfo()[vrrPolicyString];
     if (val.canConvert<uint>()) {
         return static_cast<KScreen::Output::VrrPolicy>(val.toUInt());
     }
@@ -862,12 +567,12 @@ void ControlOutput::setVrrPolicy(KScreen::Output::VrrPolicy value)
     if (infoMap.isEmpty()) {
         infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
     }
-    infoMap[QStringLiteral("vrrpolicy")] = static_cast<uint>(value);
+    infoMap[vrrPolicyString] = static_cast<uint>(value);
 }
 
 KScreen::Output::RgbRange ControlOutput::rgbRange() const
 {
-    const auto val = constInfo()[QStringLiteral("rgbrange")];
+    const auto val = constInfo()[rgbRangeString];
     if (val.canConvert<uint>()) {
         return static_cast<KScreen::Output::RgbRange>(val.toUInt());
     }
@@ -880,5 +585,5 @@ void ControlOutput::setRgbRange(KScreen::Output::RgbRange value)
     if (infoMap.isEmpty()) {
         infoMap = createOutputInfo(m_output->hashMd5(), m_output->name());
     }
-    infoMap[QStringLiteral("rgbrange")] = static_cast<uint>(value);
+    infoMap[rgbRangeString] = static_cast<uint>(value);
 }
